@@ -12,8 +12,12 @@
 #undef in_addr_t
 #undef ssize_t
 #undef socklen_t
-#undef int16_t
 #undef int32_t
+#undef int16_t
+#undef int8_t
+#undef uint32_t
+#undef uint16_t
+#undef uint8_t
 
 /* send/recvmsg cmsg types */
 #undef HAVE_CMSGHDR
@@ -82,6 +86,7 @@
 #undef LIBRARY_LIBNSL
 #undef LIBRARY_LIBRESOLV
 #undef LIBRARY_LIBSOCKET
+#undef LIBRARY_LIBRPCSOC
 #undef LIBRARY_LISTEN
 #undef LIBRARY_PATH
 #undef LIBRARY_READ
@@ -135,6 +140,7 @@
 
 /* problems on Linux */
 #undef HAVE_LINUX_BUGS
+#undef HAVE_LINUX_ECCENTRICITIES
 
 /* sun4 vsprintf doesn't seem to return length */
 #undef HAVE_BROKEN_VSPRINTF
@@ -204,6 +210,17 @@
 /* more Solaris bugs */
 #undef HAVE_SOLARIS_PAM_BUG
 
+/* support for reqtrieval of route data? */
+#undef HAVE_ROUTE_SOURCE
+
+/* architecture dependent code */
+#undef HAVE_ROUTEINFO_BSD
+#undef HAVE_ROUTEINFO_LINUX
+
+/* module defines */
+#undef HAVE_MODULE_REDIRECT
+#undef HAVE_MODULE_BANDWIDTH
+
 /* interposition.c prototypes */
 #undef HAVE_PROT_CONNECT_0
 #undef HAVE_PROT_CONNECT_1
@@ -292,13 +309,3 @@
 #undef HAVE_PROT_GETHOSTBYADDR_1
 #undef HAVE_PROT_GETHOSTBYADDR_2
 #undef HAVE_PROT_GETHOSTBYADDR_3
-
-/* XXX autoheader snafu */
-/*#undef HAVE_PROT__*/
-
-/* XXX run include/redefgen.sh manually to regenerate redefac.h after
-   changes have been made to this file */
-
-@BOTTOM@
-/* change all #undef's to #define foo 0 */
-#include "redefac.h"
