@@ -41,7 +41,7 @@
  *
  */
 
-/* $Id: common.h,v 1.276 2001/05/12 13:16:24 karls Exp $ */
+/* $Id: common.h,v 1.299 2001/11/28 10:36:24 michaels Exp $ */
 
 #ifndef _COMMON_H_
 #define _COMMON_H_
@@ -64,6 +64,20 @@
 #include "autoconf.h"
 #endif  /* HAVE_CONFIG_H */
 
+#if HAVE_LINUX_ECCENTRICITIES
+/*
+ * XXX This is a hack. Avoid transparent sockaddr union used in Linux
+ *  to avoid the use of the union in the code. Mainly used in
+ *  interposition.c (also for CMSG_)
+ */
+
+#ifdef __GNUC__
+#undef __GNUC__
+#define __GNUC__ 0
+#endif /* __GNUC__ */
+
+#endif /* HAVE_LINUX_ECCENTRICITIES */
+
 #include <sys/types.h>
 #if HAVE_SYS_TIME_H
 #include <sys/time.h>
@@ -77,29 +91,21 @@
 #if HAVE_SYS_SEM_H
 #include <sys/sem.h>
 #endif /* HAVE_SYS_SEM_H */
-/*
- * XXX This is a hack. Avoid transparent sockaddr union used in Linux
- *  to avoid the use of the union in the code. Mainly used in
- *  interposition.c
- */
-#ifdef WE_DONT_WANT_NO_SOCKADDR_ARG_UNION
-#ifdef __GNUC__
-#define __HAD_GNUC __GNUC__
-#undef __GNUC__
-#endif  /* __GNUC__ */
-#endif  /* WE_DONT_WANT_NO_SOCKADDR_ARG_UNION */
 #include <sys/socket.h>
 #include <net/if.h>
-#ifdef __HAD_GNUC
-#define __GNUC__ __HAD_GNUC
-#endif  /* __HAD_GNUC */
 #if NEED_SYS_SOCKIO_H
 #include <sys/sockio.h>
 #endif /* NEED_SYS_SOCKIO_H */
 #include <sys/mman.h>
 #include <sys/un.h>
 #include <sys/stat.h>
+#ifdef SOCKS_DLIB_OSF
+#undef __DECC
+#endif /* SOCKS_DLIB_OSF */
 #include <sys/uio.h>
+#ifdef SOCKS_DLIB_OSF
+#define __DECC
+#endif /* SOCKS_DLIB_OSF */
 #include <sys/wait.h>
 #include <netinet/in.h>
 #if HAVE_NETINET_IP_H
@@ -110,6 +116,7 @@
 #endif  /* HAVE_NETINET_IP_VAR_H */
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
+#include <sys/mman.h>
 
 #include <assert.h>
 #if HAVE_CRYPT_H
@@ -289,6 +296,38 @@ error "no known 32 bits wide datatype"
 # endif /* SIZEOF_SHORT == 4 */
 #endif /* SIZEOF_INT == 4 */
 
+#if !HAVE_INT8_T
+#define int8_t sbits_8
+#endif /* HAVE_INT8_T */
+
+#if !HAVE_INT16_T
+#define int16_t sbits_16
+#endif /* HAVE_INT16_T */
+
+#if !HAVE_INT32_T
+#define int32_t sbits_32
+#endif /* HAVE_INT32_T */
+
+#if !HAVE_UINT8_T
+#define uint8_t ubits_8
+#endif /* HAVE_UINT8_T */
+
+#if !HAVE_UINT16_T
+#define uint16_t ubits_16
+#endif /* HAVE_UINT16_T */
+
+#if !HAVE_UINT32_T
+#define uint32_t ubits_32
+#endif /* HAVE_UINT32_T */
+
+#if !HAVE_IN_PORT_T
+#define in_port_t ubits_16
+#endif /* HAVE_IN_PORT_T */
+
+#if !HAVE_IN_ADDR_T
+#define in_addr_t ubits_32
+#endif /* HAVE_IN_ADDR_T */
+
 #ifndef INADDR_NONE
 # define INADDR_NONE (ubits_32) 0xffffffff
 #endif  /* !INADDR_NONE */
@@ -332,6 +371,15 @@ error "no known 32 bits wide datatype"
 #endif
 #endif
 
+#ifndef __CONCAT3
+#if defined(__STDC__) || defined(__cplusplus)
+#define __CONCAT3(x,y,z)        x ## y ## z
+#else
+#define __CONCAT3(x,y,z)        x/**/y/**/z
+#endif
+#endif
+
+
 #if !HAVE_STRUCT_IPOPTS
 #define	MAX_IPOPTLEN	40
 struct ipoption {
@@ -393,7 +441,7 @@ struct ifaddrs {
 #define TOCIN(addr) ((const struct sockaddr_in *)(addr))
 
 /* global variables needed by everyone. */
-extern struct config_t config;
+extern struct config_t socksconfig;
 extern char *__progname;
 
 #if !HAVE_H_ERRNO
@@ -405,6 +453,8 @@ extern int h_errno;
 	 */
 
 
+#define IP_MAXPORT 65535	/* max value for ip port number. */
+
 /*
  * redefine system limits to match that of socks protocol.
  * No need for these to be bigger than protocol allows, but they
@@ -438,8 +488,8 @@ extern int h_errno;
 #define	MAXRULEADDRSTRING	 (MAXSOCKSHOSTSTRING * 2)
 
 
-#define MAXAUTHINFOLEN		((sizeof("(") - 1) + MAXMETHODSTRING) \
-									+ (sizeof(")" - 1) + (sizeof("@") - 1) + MAXNAMELEN)
+#define MAXAUTHINFOLEN		(((sizeof("(") - 1) + MAXMETHODSTRING) \
+									+ (sizeof(")") - 1) + (sizeof("@") - 1) + MAXNAMELEN)
 
 #ifndef NUL
 #define NUL '\0'
@@ -480,6 +530,21 @@ extern int h_errno;
 
 #define close(n)	closen(n)
 
+/* XXX needed on AIX apparently */
+#ifdef recvmsg
+#define recvmsg_system recvmsg
+#undef recvmsg
+#endif /* recvmsg */
+
+#define recvmsg(s, msg, flags)	recvmsgn(s, msg, flags)
+
+#ifdef sendmsg
+#define sendmsg_system sendmsg
+#undef sendmsg
+#endif /* sendmsg */
+
+#define sendmsg(s, msg, flags)	sendmsgn(s, msg, flags)
+
 #define PORTISRESERVED(port)	\
 	(ntohs((port)) != 0 && ntohs((port)) < IPPORT_RESERVED)
 
@@ -500,95 +565,124 @@ extern int h_errno;
  * macros to manipulate ancillary data depending on if we're on sysv or BSD.
  */
 
-/* allocate memory for data.  "size" is the amount of memory to allocate. */
+/*
+ * allocate memory for a controlmessage of size "size".  "name" is the
+ * name of the allocated memory.
+ */ 
 #if HAVE_CMSGHDR
-#define CMSG_AALLOC(size) \
+#define CMSG_AALLOC(name, size) \
 	union { \
 		char cmsgmem[sizeof(struct cmsghdr) + (size)]; \
 		struct cmsghdr align; \
-	} cmsgmem; \
-	struct cmsghdr *cmsg = &cmsgmem.align
+	} __CONCAT3(_, name, mem); \
+	struct cmsghdr *name = &__CONCAT3(_, name, mem).align
 #else /* !HAVE_CMSGHDR */
-#define CMSG_AALLOC(size) \
-	char cmsgmem[(size)]
+#define CMSG_AALLOC(name, size) \
+	char name[(size)]
 #endif /* !HAVE_CMSGHDR */
 
-
 /*
- * add a object to data.  "object" is the object to add to data at
- * offset "offset".
+ * Returns the size of the previously allocated controlmessage named
+ * "name"
  */
 #if HAVE_CMSGHDR
-#define CMSG_ADDOBJECT(object, offset) \
+#define CMSG_MEMSIZE(name) (sizeof(__CONCAT3(_, name, mem)))
+#else /* !HAVE_CMSGHDR */
+#define CMSG_MEMSIZE(name) (sizeof((name)))
+#endif
+
+/*
+ * Returns the controldata member of "msg".
+ */
+#if HAVE_CMSGHDR
+/* cast is neccessary on AIX, due to buggy headers there?. */
+#define CMSG_CONTROLDATA(msg)	((struct cmsghdr *)((msg).msg_control))
+#else /* !HAVE_CMSGHDR */
+#define CMSG_CONTROLDATA(msg)	((msg).msg_accrights)
+#endif
+
+/*
+ * add "object" to "data".  "object" is the object to add to "data" at
+ * offset "offset".  
+ */
+#if HAVE_CMSGHDR
+#define CMSG_ADDOBJECT(object, data, offset) \
 	do \
-		memcpy(CMSG_DATA(cmsg) + (offset), &(object), sizeof(object)); \
+		memcpy(CMSG_DATA(data) + (offset), &(object), sizeof(object)); \
 	while (lintnoloop_common_h)
 #else /* !HAVE_CMSGHDR */
-#define CMSG_ADDOBJECT(object, offset) \
+#define CMSG_ADDOBJECT(object, data, offset) \
 	do \
-		memcpy(cmsgmem + (offset), &(object), sizeof((object))); \
+		memcpy(data + (offset), &(object), sizeof((object))); \
 	while (lintnoloop_common_h)
 #endif /* !HAVE_CMSGHDR */
 
 
 /*
- * get a object from data.  "object" is the object to get from data at
- * offset "offset".
+ * get a object from controldata "data". 
+ * "object" is the object to fill with data gotten from "data" at offset
+ * "offset".
  */
 #if HAVE_CMSGHDR
-#define CMSG_GETOBJECT(object, offset) \
+#define CMSG_GETOBJECT(object, data, offset) \
 	do \
-		memcpy(&(object), CMSG_DATA(cmsg) + (offset), sizeof((object))); \
+		memcpy(&(object), CMSG_DATA((data)) + (offset), sizeof((object))); \
 	while (lintnoloop_common_h)
 #else /* !HAVE_CMSGHDR */
-#define CMSG_GETOBJECT(object, offset) \
+#define CMSG_GETOBJECT(object, data, offset) \
 	do \
-		memcpy(&(object), cmsgmem + (offset), sizeof((object))); \
+		memcpy(&(object), ((data) + (offset)), sizeof((object))); \
 	while (lintnoloop_common_h)
 #endif /* !HAVE_CMSGHDR */
 
 
 
-/* set cmsg for sending */
+/*
+ * Sets up "object" for sending a controlmessage of size "size".
+ * "controlmem" is the memory the controlmessage is stored in.
+ */
 #if HAVE_CMSGHDR
-#define CMSG_SETHDR_SEND(size) \
+#define CMSG_SETHDR_SEND(object, controlmem, size) \
 	do { \
-		cmsg->cmsg_level		= SOL_SOCKET; \
-		cmsg->cmsg_type		= SCM_RIGHTS; \
-		cmsg->cmsg_len			= sizeof(struct cmsghdr) + (size); \
+		controlmem->cmsg_level		= SOL_SOCKET; \
+		controlmem->cmsg_type		= SCM_RIGHTS; \
+		controlmem->cmsg_len			= sizeof(struct cmsghdr) + (size); \
 		\
-		msg.msg_control		= (caddr_t)cmsg; \
-		msg.msg_controllen	= cmsg->cmsg_len; \
+		object.msg_control		= (caddr_t)controlmem; \
+		object.msg_controllen	= controlmem->cmsg_len; \
 	} while (lintnoloop_common_h)
 #else /* !HAVE_CMSGHDR */
-#define CMSG_SETHDR_SEND(size) \
+#define CMSG_SETHDR_SEND(object, controlmem, size) \
 	do { \
-		msg.msg_accrights		= (caddr_t)cmsgmem; \
-		msg.msg_accrightslen	= (size); \
+		object.msg_accrights		= (caddr_t)controlmem; \
+		object.msg_accrightslen	= (size); \
 	} while (lintnoloop_common_h)
 #endif /* !HAVE_CMSGHDR */
 
-/* set cmsg for receiving */
+/*
+ * Sets up "object" for receiving a controlmessage of size "size".
+ * "controlmem" is the memory set asside for the controlmessage.
+ */
 #if HAVE_CMSGHDR
-#define CMSG_SETHDR_RECV(size) \
+#define CMSG_SETHDR_RECV(object, controlmem, size) \
 	do { \
-		msg.msg_control		= (caddr_t)cmsg; \
-		msg.msg_controllen	= (size); \
+		object.msg_control		= (caddr_t)controlmem; \
+		object.msg_controllen	= (size); \
 	} while (lintnoloop_common_h)
 #else /* !HAVE_CMSGHDR */
-#define CMSG_SETHDR_RECV(size) \
+#define CMSG_SETHDR_RECV(object, controlmem, size) \
 	do { \
-		msg.msg_accrights		= (caddr_t)cmsgmem; \
-		msg.msg_accrightslen	= (size); \
+		object.msg_accrights		= (caddr_t)controlmem; \
+		object.msg_accrightslen	= (size); \
 	} while (lintnoloop_common_h)
 #endif /* !HAVE_CMSGHDR */
 
 
 /* returns length of controldata actually sent. */
 #if HAVE_CMSGHDR
-#define CMSG_GETLEN(msg)	(msg.msg_controllen - sizeof(struct cmsghdr))
+#define CMSG_GETLEN(msg)	((msg).msg_controllen - sizeof(struct cmsghdr))
 #else
-#define CMSG_GETLEN(msg)	(msg.msg_accrightslen)
+#define CMSG_GETLEN(msg)	((msg).msg_accrightslen)
 #endif
 
 
@@ -707,10 +801,14 @@ do {														\
 #define FAKEIP_END	0x000000ff
 
 #define SOCKS_V4					4
+#define SOCKS_V4s					"socks v4"
 #define SOCKS_V4REPLY_VERSION 0
 #define SOCKS_V5					5
+#define SOCKS_V5s					"socks v5"
 #define MSPROXY_V2				2
+#define MSPROXY_V2s				"msproxy v2"
 #define HTTP_V1_0					1
+#define HTTP_V1_0s				"http v1.0"
 
 /* subnegotiation. */
 #define SOCKS_UNAMEVERSION		1
@@ -905,20 +1003,15 @@ do {														\
 
 enum operator_t { none = 0, eq, neq, ge, le, gt, lt, range };
 
-struct compat_t {
-	unsigned reuseaddr:1;				/* set SO_REUSEADDR?								*/
-	unsigned sameport:1;					/* always try to use same port as client?	*/
-	unsigned :0;
-};
-
 
 struct logtype_t {
 	int				type;			/* type of logging (where to).						*/
 	FILE				**fpv;		/* if logging is to file, this is the open file.*/
-	int				fpc;
+	char				**fnamev;	/* if logging is to file, name of file.			*/
+	size_t			fpc;			/* number of files.										*/
 	int				*fplockv;	/* locking of logfiles.									*/
 	int				facility;	/* if logging to syslog, this is the facility.	*/
-};
+	const char		*facilityname;	/* if logging to syslog, name of facility.	*/ };
 
 
 
@@ -1420,12 +1513,12 @@ struct socksfd_t {
 	struct sockaddr		remote;		/* address server is using on our behalf.	*/
 	unsigned					:0;
 	struct sockaddr		reply;		/* address to expect reply from.				*/
+	unsigned					:0;
 
-	/* XXX union this. */
-	unsigned					:0;
-	struct sockaddr		accepted;	/* address server accepted for us.			*/
-	unsigned					:0;
-	struct sockaddr		connected;	/* address server connected to for us.		*/
+	union {
+		struct sockaddr		accepted;	/* address server accepted for us.		*/
+		struct sockaddr		connected;	/* address server connected to for us.	*/
+	} forus;
 
 	struct route_t		*route;
 };
@@ -1499,29 +1592,6 @@ udpheader_add __P((const struct sockshost_t *host, char *msg, size_t *len,
  *		On failure: NULL (out of memory).
  */
 
-struct udpheader_t *
-string2udpheader __P((const char *data, size_t len,
-							 struct udpheader_t *header));
-/*
- * Converts "data" to udpheader_t representation.
- * "len" is length of "data".
- * "data" is assumed to be in network order.
- * Returns:
- *		On success: pointer to a udpheader_t in static memory.
- *		On failure: NULL ("data" is not a complete udppacket).
- */
-
-
-const char *
-socks_packet2string __P((const void *packet, int type));
-/*
- * debug function; dumps socks packet content
- * "packet" is a socks packet, "type" indicates it's type.
- * Returns:
- *		On success: 0
- *		On failure: -1
- */
-
 int
 socks_socketisbound __P((int s));
 /*
@@ -1552,16 +1622,6 @@ socks_logmatch __P((unsigned int d, const struct logtype_t *log));
  * Returns false otherwise.
  */
 
-char *
-sockaddr2string __P((const struct sockaddr *address, char *string, size_t len));
-/*
- * Returns the IP address and port in "address" on string form.
- * "address" is assumed to be on network form and it will be
- * converted to host form before written to "string".
- * "len" gives length of the NUL terminated string.
- * Returns: "string".
- */
-
 
 struct sockaddr *
 sockshost2sockaddr __P((const struct sockshost_t *shost,
@@ -1608,17 +1668,26 @@ sockshost2ruleaddress __P((const struct sockshost_t *host,
 
 struct ruleaddress_t *
 sockaddr2ruleaddress __P((const struct sockaddr *addr,
-							struct ruleaddress_t *ruleaddr));
+								  struct ruleaddress_t *ruleaddr));
 /*
- * Converts the struct sockaddr "addr" to a ruleaddress_t struct and stores it
- * in "ruleaddr".
+ * Converts the struct sockaddr "addr" to a ruleaddress_t struct and stores
+ * it in "ruleaddr".
  * Returns: "addr".
  */
 
 struct sockaddr *
-ifname2sockaddr __P((const char *ifname, struct sockaddr *addr));
+hostname2sockaddr __P((const char *name, int index, struct sockaddr *addr));
 /*
- * Finds the first address found on the interface named "ifname".
+ * Retrieves the address with index "index" for the hostname named "name".
+ * Returns:
+ *		On success: "addr", filled in with the address found.
+ *		On failure: NULL (no address found).
+ */
+
+struct sockaddr *
+ifname2sockaddr __P((const char *ifname, int index, struct sockaddr *addr));
+/*
+ * Retrieves the address with index "index" on the interface named "ifname".
  * Returns:
  *		On success: "addr", filled in with the address found.
  *		On failure: NULL (no address found).
@@ -1632,11 +1701,15 @@ sockatmark __P((int s));
  */
 
 ssize_t
-recvmsgn __P((int s, struct msghdr *msg, int flags, size_t len));
+recvmsgn __P((int s, struct msghdr *msg, int flags));
 /*
- * Like recvmsg(), but tries to read until "len" has been read.
- * BUGS:
- *   Assumes msg->msg_iov[n] are laid out next to each others.
+ * Like recvmsg(), but tries to read until all has been read.
+ */
+
+ssize_t
+sendmsgn __P((int s, const struct msghdr *msg, int flags));
+/*
+ * Like sendmsg(), but tries to send until all has been sent.
  */
 
 ssize_t
@@ -1701,15 +1774,6 @@ snprintfn();
  *		never returns a value greater than size - 1.
  */
 
-char *
-sockshost2string __P((const struct sockshost_t *host, char *string,
-							 size_t len));
-/*
- * Writes "host" out as a string.  The string is written to "string",
- * which is of length "len", including NUL termination.
- * Returns: "string".
- */
-
 const char *
 strcheck __P((const char *string));
 /*
@@ -1717,37 +1781,6 @@ strcheck __P((const char *string));
  * exhausted, if not, returns the same string it was passed.
  */
 
-const char *
-command2string __P((int command));
-/*
- * Returns a printable representation of the socks command "command".
- * Can't fail.
- */
-
-const char *
-protocol2string __P((int protocol));
-/*
- * Returns a printable representation of "protocol".
- * Can't fail.
- */
-
-
-
-const char *
-method2string __P((int method));
-/*
- * Returns a printable representation of the authmethod "method".
- * Can't fail.
- */
-
-int
-string2method __P((const char *methodname));
-/*
- * If "methodname" is the name of a supported method, the protocol
- * value of that method is returned.
- * Otherwise, -1 is returned.
- */
-
 
 unsigned char *
 sockshost2mem __P((const struct sockshost_t *host, unsigned char *mem,
@@ -1939,15 +1972,6 @@ socks_getroute __P((const struct request_t *req, const struct sockshost_t *src,
  *		On failure: NULL (no socks route found).
  */
 
-const char *
-ruleaddress2string __P((const struct ruleaddress_t *rule, char *string,
-								size_t len));
-/*
- * Writes "rule" out as a string.  The string is written to "string",
- * which is of length "len", including NUL termination.
- * Returns: "string".
- */
-
 
 unsigned char
 sockscode __P((int version, int code));
@@ -1962,20 +1986,6 @@ errno2reply __P((int errnum, int version));
  * Returns the socks version "version" reply code for a error of type "errno".
  */
 
-enum operator_t
-string2operator __P((const char *operator));
-/*
- * Returns the enum for the string representation of a operator.
- * Can't fail.
- */
-
-const char *
-operator2string __P((enum operator_t operator));
-/*
- * Returns the string representation of the operator.
- * Can't fail.
- */
-
 char *
 str2vis __P((const char *string, size_t len));
 /*
@@ -2100,6 +2110,12 @@ socks_unlock __P((int d));
  * Unlocks the filedescriptor "d", previously locked by this process.
  */
 
+int
+bitcount __P((unsigned long number));
+/*
+ * Returns the number of bits set in "number".
+ */
+
 #if SOCKSLIBRARY_DYNAMIC
 struct hostent *sys_gethostbyaddr __P((const char *addr, int len, int af));
 struct hostent *sys_gethostbyname __P((const char *));
@@ -2201,6 +2217,13 @@ socks_getpwnam __P((const char *login));
  * password too.
  */
 
+void
+checkmodule __P((const char *name));
+/*
+ * Checks that the system has the module "name" and permission to use it.
+ * Aborts with a errormessage if not.
+ */
+
 __END_DECLS
 
 #if SOCKSLIBRARY_DYNAMIC
@@ -2216,3 +2239,5 @@ __END_DECLS
 #include "sockd.h"
 #endif  /* SOCKS_CLIENT */
 #endif  /* SOCKS_CLIENT || SOCKS_SERVER */
+
+#include "tostring.h"
