@@ -41,7 +41,7 @@
  *
  */
 
-/* $Id: config.h,v 1.45 2001/05/15 13:45:30 karls Exp $ */
+/* $Id: config.h,v 1.47 2001/11/22 14:00:40 michaels Exp $ */
 
 #ifndef _CONFIG_H_
 #define _CONFIG_H_
@@ -72,7 +72,7 @@
 #define SOCKS_LOCKFILE				"./sockslockXXXXXXXXXX"
 
 /* if profiling is enabled, directory to store profile files in. */
-#define SOCKS_PROFILEDIR			"./prof"
+#define SOCKS_PROFILEDIR			"./.prof"
 
 
 	/*
@@ -202,7 +202,11 @@
  * You can probably set this to a big number.
  * Each client will occupy one descriptor.
  */
+#if DEBUG
+#define SOCKD_NEGOTIATEMAX			2
+#else
 #define SOCKD_NEGOTIATEMAX			24
+#endif /* DEBUG */
 
 /*
  * max number of clients each i/o process will handle.
@@ -212,7 +216,11 @@
  * from doing any i/o untill a i/o slot has become available.  It is
  * therefore important that enough i/o slots are available at all times.
  */
+#if DEBUG
+#define SOCKD_IOMAX					2
+#else
 #define SOCKD_IOMAX					8
+#endif /* DEBUG */
 
 /* server buffersize for network i/o using TCP. */
 #define SOCKD_BUFSIZETCP			(1024 * 16)
