@@ -45,7 +45,7 @@
 #include "config_parse.h"
 
 static const char rcsid[] =
-"$Id: sockd_io.c,v 1.184 2001/05/02 10:57:16 michaels Exp $";
+"$Id: sockd_io.c,v 1.203 2001/11/23 12:44:30 karls Exp $";
 
 /*
  * Accept io objects from mother and does io on them.  We never
@@ -78,10 +78,11 @@ io_finddescriptor __P((int d));
  */
 
 static int
-io_fillset __P((fd_set *set, int antiflags));
+io_fillset __P((fd_set *set, int antiflags, const struct timeval *timenow));
 /*
  * Sets all descriptors in our list in the set "set".  If "flags"
  * is set, io's with any of the flags in "flags" set will be excluded.
+ * "timenow" is the time now.
  * Returns the highest descriptor in our list, or -1 if we don't
  * have any descriptors open currently.
  */
@@ -107,13 +108,12 @@ doio __P((int mother, struct sockd_io_t *io, fd_set *rset, fd_set *wset,
 
 static int
 io_rw __P((struct sockd_io_direction_t *in, struct sockd_io_direction_t *out,
-			  int *bad, char *buf, int flags));
+			  int *bad, void *buf, size_t bufsize, int flags));
 /*
  * Transfers data from "in" to "out" using flag "flags".
  * "inauth" is the authentication used for reading from "in",
  * "outauth" is the authentication * used when writing to out.
- * The data transfered uses "buf" as a buffer, which must be big
- * enough to hold the data transfered.
+ * The data transfered uses "buf" as a buffer, which is of size "bufsize".
  * The function never transfers more than the receive low watermark
  * of "out".
  *
@@ -146,7 +146,7 @@ proctitleupdate __P((void));
  */
 
 static struct timeval *
-io_gettimeout __P((struct timeval *timeout));
+io_gettimeout __P((struct timeval *timeout, const struct timeval *timenow));
 /*
  * If there is a timeout on the current clients for how long to exist
  * without doing i/o, this function fills in "timeout" with the appropriate
@@ -157,10 +157,10 @@ io_gettimeout __P((struct timeval *timeout));
  */
 
 static struct sockd_io_t *
-io_gettimedout __P((void));
+io_gettimedout __P((const struct timeval *timenow));
 /*
- * Scans all clients for one that has timed out according to config
- * settings.
+ * Scans all clients for one that has timed out according to socksconfig
+ * settings. "timenow" is the time now.
  * Returns:
  *		If timed out client found: pointer to it.
  *		Else: NULL.
@@ -198,11 +198,26 @@ siginfo __P((int sig));
 		} \
 	} while (lintnoloop_sockd_h)
 
+
+/* timersub macro from OpenBSD */
+#define timersub_hack(tvp, uvp, vvp)                                    \
+        do {                                                            \
+                (vvp)->tv_sec = (tvp)->tv_sec - (uvp)->tv_sec;          \
+                (vvp)->tv_usec = (tvp)->tv_usec - (uvp)->tv_usec;       \
+                if ((vvp)->tv_usec < 0) {                               \
+                        (vvp)->tv_sec--;                                \
+                        (vvp)->tv_usec += 1000000;                      \
+                }                                                       \
+        } while (0)
+
+
+
 __END_DECLS
 
 
 static struct sockd_io_t iov[SOCKD_IOMAX];	/* each child has these io's. */
 static int ioc = ELEMENTS(iov);
+static struct timeval bwoverflow;
 
 void
 run_io(mother)
@@ -232,14 +247,19 @@ run_io(mother)
 		int rbits, bits;
 		fd_set rset, wset, xset, newrset, controlset, tmpset;
 		struct sockd_io_t *io;
-		struct timeval timeout;
+		struct timeval timeout, timenow;
+
+		gettimeofday(&timenow, NULL);
 
 		/* look for timed-out clients. */
-		while ((io = io_gettimedout()) != NULL)
+		while ((io = io_gettimedout(&timenow)) != NULL)
 			delete_io(mother->ack, io, -1, IO_TIMEOUT);
 
-		io_fillset(&xset, MSG_OOB);
-		rbits = io_fillset(&rset, 0);
+		/* starting a new run. */
+		timerclear(&bwoverflow);
+
+		io_fillset(&xset, MSG_OOB, &timenow);
+		rbits = io_fillset(&rset, 0, &timenow);
 
 		if (mother->s != -1) {
 			FD_SET(mother->s, &rset);
@@ -258,7 +278,8 @@ run_io(mother)
 		 * the i/o function if there's one pending later.
 		 */
 		++rbits;
-		switch (selectn(rbits, &rset, NULL, &xset, io_gettimeout(&timeout))) {
+		switch (selectn(rbits, &rset, NULL, &xset,
+		io_gettimeout(&timenow, &timeout))) {
 			case -1:
 				SERR(-1);
 				/* NOTREACHED */
@@ -296,8 +317,10 @@ run_io(mother)
 		 * we have reason to care about them.
 		 */
 
+		gettimeofday(&timenow, NULL);
+
 		/* descriptors to check for readability; those not already set. */
-		bits = io_fillset(&tmpset, 0);
+		bits = io_fillset(&tmpset, 0, &timenow);
 		bits = fdsetop(bits + 1, '^', &rset, &tmpset, &newrset);
 		if (mother->s != -1) { /* mother status may change too. */
 			FD_SET(mother->s, &newrset);
@@ -345,7 +368,8 @@ run_io(mother)
 			continue;
 		}
 
-		switch (selectn(bits, &newrset, &wset, NULL, io_gettimeout(&timeout))) {
+		switch (selectn(bits, &newrset, &wset, NULL,
+		io_gettimeout(&timeout, &timenow))) {
 			case -1:
 				SERR(-1);
 				/* NOTREACHED */
@@ -436,7 +460,11 @@ delete_io(mother, io, fd, status)
 
 	SASSERTX(io->allocated);
 
-	/* if client or socks-rules specifies logging disconnect, log once. */
+	if (io->state.protocol == SOCKS_TCP) /* udp rules are temporary. */
+		/* request handled. */
+		bwfree(io->rule.bw);
+
+	/* if client or socks-rules specifies logging disconnect, log, but once. */
 	if (io->rule.log.disconnect)
 		rule = &io->rule;
 	else if (io->acceptrule.log.disconnect)
@@ -445,6 +473,7 @@ delete_io(mother, io, fd, status)
 		rule = NULL;
 
 	if (rule != NULL && rule->log.disconnect) {
+		/* LINTED constant in conditional context */
 		char in[MAXSOCKADDRSTRING + MAXAUTHINFOLEN];
 		char out[sizeof(in)];
 		char logmsg[sizeof(in) + sizeof(out) + 1024];
@@ -552,6 +581,7 @@ delete_io(mother, io, fd, status)
 	}
 
 	close_iodescriptors(io);
+
 	io->allocated = 0;
 
 	if (mother != -1) {
@@ -559,7 +589,7 @@ delete_io(mother, io, fd, status)
 
 		/* ack io slot free. */
 		if (writen(mother, &b, sizeof(b), NULL) != sizeof(b))
-			swarn("%s: writen(): mother", function);
+			 swarn("%s: writen(): mother", function);
 	}
 
 	proctitleupdate();
@@ -604,7 +634,7 @@ recv_io(s, io)
 	size_t length = 0;
 	struct iovec iovec[1];
 	struct msghdr msg;
-	CMSG_AALLOC(sizeof(int) * FDPASS_MAX);
+	CMSG_AALLOC(cmsg, sizeof(int) * FDPASS_MAX);
 
 	if (io == NULL) {	/* child semantics; find a io ourselves. */
 		for (i = 0; i < ioc; ++i)
@@ -636,10 +666,10 @@ recv_io(s, io)
 	msg.msg_namelen		= 0;
 
 	/* LINTED pointer casts may be troublesome */
-	CMSG_SETHDR_RECV(sizeof(cmsgmem));
+	CMSG_SETHDR_RECV(msg, cmsg, CMSG_MEMSIZE(cmsg));
 
-	if (recvmsgn(s, &msg, 0, length) != (ssize_t)length) {
-		swarn("%s: recvmsgn()", function);
+	if (recvmsg(s, &msg, 0) != (ssize_t)length) {
+		swarn("%s: recvmsg()", function);
 		return -1;
 	}
 
@@ -678,16 +708,17 @@ recv_io(s, io)
 	fdreceived = 0;
 
 	/* LINTED pointer casts may be troublesome */
-	CMSG_GETOBJECT(io->src.s, sizeof(io->src.s) * fdreceived++);
+	CMSG_GETOBJECT(io->src.s, cmsg, sizeof(io->src.s) * fdreceived++);
 	/* LINTED pointer casts may be troublesome */
-	CMSG_GETOBJECT(io->dst.s, sizeof(io->dst.s) * fdreceived++);
+	CMSG_GETOBJECT(io->dst.s, cmsg, sizeof(io->dst.s) * fdreceived++);
 
 	switch (io->state.command) {
 		case SOCKS_BIND:
 		case SOCKS_BINDREPLY:
 			if (io->state.extension.bind)
 				/* LINTED pointer casts may be troublesome */
-				CMSG_GETOBJECT(io->control.s, sizeof(io->control.s) * fdreceived++);
+				CMSG_GETOBJECT(io->control.s, cmsg,
+				sizeof(io->control.s) * fdreceived++);
 			else
 				io->control.s = -1;
 			break;
@@ -698,17 +729,18 @@ recv_io(s, io)
 
 		case SOCKS_UDPASSOCIATE:
 			/* LINTED pointer casts may be troublesome */
-			CMSG_GETOBJECT(io->control.s, sizeof(io->control.s) * fdreceived++);
+			CMSG_GETOBJECT(io->control.s, cmsg,
+			sizeof(io->control.s) * fdreceived++);
 			break;
 
 		default:
 			SERRX(io->state.command);
 	}
 
-	time(&io->time);
+	gettimeofday(&io->time, NULL);
 	io->allocated = 1;
 
-#if DEBUG
+#if HARDCORE_DEBUG
 	printfd(io, "received");
 #endif
 
@@ -770,6 +802,8 @@ doio(mother, io, rset, wset, flags)
 	char buf[MAX(SOCKD_BUFSIZETCP, SOCKD_BUFSIZEUDP)
 	+ sizeof(struct udpheader_t)];
 	ssize_t r, w;
+	size_t bwused;
+	struct timeval timenow;
 
 
 	SASSERTX(io->allocated);
@@ -779,14 +813,44 @@ doio(mother, io, rset, wset, flags)
 	||			(flags & MSG_OOB)
 	||			(io->control.s != -1 && FD_ISSET(io->control.s, rset)));
 
+	/*
+	 * we are only called when we have i/o to do.
+	 * Could probably remove this gettimeofday() call to, but there are
+	 * platforms without SO_SNDLOWAT which prevents us.
+	 */
+	gettimeofday(&timenow, NULL);
+
+	bwused = 0;
 	switch (io->state.protocol) {
 		case SOCKS_TCP: {
 			int bad;
+			size_t bufsize;
 
-			/* from in to out... */
+			if (io->rule.bw != NULL) {
+				ssize_t left;
+
+				if ((left = bwleft(io->rule.bw)) <= 0) {
+					/* 
+					 * update data (new time) so next bwleft() presumably
+					 * has some left.
+					 * No harm in calling bwupdate() without le 0 check, but
+					 * maybe this is smarter (avaids extra lock in gt 0 case).
+					 */
+					bwupdate(io->rule.bw, 0, &timenow);
+					left = bwleft(io->rule.bw);
+				}
+
+				if ((bufsize = MIN(sizeof(buf), (size_t)left)) == 0)
+					break;
+			}
+			else
+				bufsize = sizeof(buf);
+
+
+			/* from in to out ... */
 			if (FD_ISSET(io->src.s, rset) && FD_ISSET(io->dst.s, wset)) {
 				bad = -1;
-				r = io_rw(&io->src, &io->dst, &bad, buf, flags);
+				r = io_rw(&io->src, &io->dst, &bad, buf, bufsize, flags);
 				if (bad != -1) {
 					delete_io(mother, io, bad, r);
 					return;
@@ -794,12 +858,31 @@ doio(mother, io, rset, wset, flags)
 
 				iolog(&io->rule, &io->state, OPERATION_IO, &io->src.host,
 				&io->src.auth, &io->dst.host, &io->dst.auth, buf, (size_t)r);
+				
+				bwused += r;
 			}
 
 			/* ... and out to in. */
+#if 0
+			/*
+			 * This doesn't work too good since we can end up doing i/o
+			 * only in -> out for a long time.  Also since we assume one
+			 * side is on the lan (where b/w isn't that critical)
+			 * and the other side is the net, assume some slack on
+			 * one side is ok.  Same applices to udp case.
+			 * Another option would be to alternate which direction we
+			 * do i/o on first each time, but we instead do the simple 
+			 * thing and just don't subtract bufsize.
+			 */
+			bufsize = ...
+#endif
+
+			if (bufsize == 0)
+				break;
+
 			if (FD_ISSET(io->dst.s, rset) && FD_ISSET(io->src.s, wset)) {
 				bad = -1;
-				r = io_rw(&io->dst, &io->src, &bad, buf, flags);
+				r = io_rw(&io->dst, &io->src, &bad, buf, bufsize, flags);
 				if (bad != -1) {
 					delete_io(mother, io, bad, r);
 					return;
@@ -807,6 +890,8 @@ doio(mother, io, rset, wset, flags)
 
 				iolog(&io->rule, &io->state, OPERATION_IO, &io->dst.host,
 				&io->dst.auth, &io->src.host, &io->src.auth, buf, (size_t)r);
+
+				bwused += r;
 			}
 
 			break;
@@ -823,6 +908,11 @@ doio(mother, io, rset, wset, flags)
 			 * was received from expected in.host, etc.
 			 */
 
+			/* 
+			 * We are less strict about it in the udp case since we don't 
+			 * want to truncate packets.
+			 */
+
 			/* UDP to relay from client to destination? */
 			if (FD_ISSET(io->src.s, rset) && FD_ISSET(io->dst.s, wset)) {
 				const int lflags = flags & ~MSG_OOB;
@@ -838,7 +928,7 @@ doio(mother, io, rset, wset, flags)
 
 				/*
 				 * If client hasn't sent us it's address yet we have to
-				 * assume the first packet is from is it.
+				 * assume the first packet is from it.
 				 * Client can only blame itself if not.
 				 */
 
@@ -928,6 +1018,8 @@ doio(mother, io, rset, wset, flags)
 				&io->control.laddr, &io->rule, &io->state, &io->src.host,
 				&io->dst.host, NULL, 0);
 
+				bwuse(io->rule.bw);
+
 				/* set r to bytes sent by client sans socks UDP header. */
 				r -= PACKETSIZE_UDP(&header);
 
@@ -938,8 +1030,14 @@ doio(mother, io, rset, wset, flags)
 				if (!permit)
 					break;
 
+				if (redirect(io->dst.s, &io->dst.laddr, &io->dst.host,
+				io->state.command, &io->rule.rdr_from, &io->rule.rdr_to) != 0) {
+					swarn("%s: redirect()", function);
+					break;
+				}
+
 				/* LINTED pointer casts may be troublesome */
-				sockshost2sockaddr(&header.host, &io->dst.raddr);
+				sockshost2sockaddr(&io->dst.host, &io->dst.raddr);
 
 				/* LINTED pointer casts may be troublesome */
 				if ((w = socks_sendto(io->dst.s, &buf[PACKETSIZE_UDP(&header)],
@@ -947,7 +1045,9 @@ doio(mother, io, rset, wset, flags)
 				&io->dst.auth)) != r)
 					iolog(&io->rule, &io->state, OPERATION_ERROR, &io->src.host,
 					&io->src.auth, &io->dst.host, &io->dst.auth, NULL, 0);
+
 				io->dst.written += MAX(0, w);
+				bwused += MAX(0, w);
 			}
 
 
@@ -961,17 +1061,24 @@ doio(mother, io, rset, wset, flags)
 			 * We only peek enough to get the source but this still involves
 			 * an extra systemcall.  Can we find a better/faster way to do it?
 			 */
+
+#if 0 /* see comment for tcp case. */
+			if (bwused >= io->rule.bw->maxbps)
+				break;
+#endif
+
 			if (FD_ISSET(io->dst.s, rset) && FD_ISSET(io->src.s, wset)) {
 				const int lflags = flags & ~MSG_OOB;
 				struct connectionstate_t state;
-				struct sockaddr from;
-				struct sockshost_t srcsh;
+				struct sockaddr rfrom;
+				struct sockshost_t rfromhost, replyto;
 				char *newbuf;
+				int s = io->src.s;
 
 				/* MSG_PEEK because of libwrap, see above. */
-				fromlen = sizeof(from);
-				if ((r = socks_recvfrom(io->dst.s, buf, 1, lflags | MSG_PEEK, &from,
-				&fromlen, &io->dst.auth)) == -1) {
+				fromlen = sizeof(rfrom);
+				if ((r = socks_recvfrom(io->dst.s, buf, 1, lflags | MSG_PEEK,
+				&rfrom, &fromlen, &io->dst.auth)) == -1) {
 					delete_io(mother, io, io->dst.s, r);
 					return;
 				}
@@ -993,50 +1100,79 @@ doio(mother, io, rset, wset, flags)
 
 				/* LINTED possible pointer alignment problem */
 				if (io->dst.host.atype == SOCKS_ADDR_DOMAIN
-				&& sockaddrareeq(&io->dst.raddr, &from))
-					srcsh = io->dst.host;
+				&& sockaddrareeq(&io->dst.raddr, &rfrom))
+					rfromhost = io->dst.host;
 				else
-					sockaddr2sockshost(&from, &srcsh);
+					sockaddr2sockshost(&rfrom, &rfromhost);
 
 				/* only set temporary here for one replypacket at a time. */
 				state				= io->state;
 				state.command	= SOCKS_UDPREPLY;
 
 				permit = rulespermit(io->dst.s, &io->control.raddr,
-				&io->control.laddr, &io->rule, &state, &srcsh, &io->src.host,
-				NULL, 0);
+				&io->control.laddr, &io->rule, &state, &rfromhost,
+				&io->src.host, NULL, 0);
+
+				bwuse(io->rule.bw);
 
 				io->dst.auth = io->state.auth;
 
 				/* read the peeked packet out of the buffer. */
-				fromlen = sizeof(from);
+				fromlen = sizeof(rfrom);
 				if ((r = socks_recvfrom(io->dst.s, buf, io->src.sndlowat, lflags,
-				&from, &fromlen, &io->dst.auth)) == -1) {
+				&rfrom, &fromlen, &io->dst.auth)) == -1) {
 					delete_io(mother, io, io->dst.s, r);
 					return;
 				}
 				io->dst.read += r;
+				bwused += r;
 
-				iolog(&io->rule, &state, OPERATION_IO, &srcsh, &io->dst.auth,
-				&io->src.host, &io->src.auth, buf, (size_t)r);
+				iolog(&io->rule, &state, OPERATION_IO, &rfromhost,
+				&io->dst.auth, &io->src.host, &io->src.auth, buf, (size_t)r);
 
 				if (!permit)
 					break;
 
+				replyto = io->src.host;
+				if (redirect(io->src.s, &rfrom, &replyto, 
+				state.command, &io->rule.rdr_from, &io->rule.rdr_to) != 0) {
+					swarn("%s: redirect()", function);
+					break;
+				}
+				
+				if (!sockshostareeq(&replyto, &io->src.host)) {
+					/* need to redirect reply. */
+					if ((s = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
+						swarn("%s: socket()", function);
+						break;
+					}
+						
+					if (socks_connect(s, &replyto) != 0) {
+						swarn("%s: socks_connect()", function);
+						break;
+					}
+				}
+
+				/* in case redirect() changed it . */
+				sockaddr2sockshost(&rfrom, &rfromhost);
+
 				/* add socks UDP header.  */
 				/* LINTED pointer casts may be troublesome */
-				newbuf = udpheader_add(&srcsh, buf, (size_t *)&r, sizeof(buf));
+				newbuf = udpheader_add(&rfromhost, buf, (size_t *)&r, sizeof(buf));
 				SASSERTX(newbuf == buf);
 
 				/*
 				 * XXX socket must be connected but that should always be the
 				 * case for now since binding UDP addresses is not supported.
 				 */
-				if ((w = socks_sendto(io->src.s, newbuf, (size_t)r, lflags, NULL, 0,
+				if ((w = socks_sendto(s, newbuf, (size_t)r, lflags, NULL, 0,
 				&io->src.auth)) != r)
-					iolog(&io->rule, &state, OPERATION_ERROR, &srcsh, &io->dst.auth,
-					&io->src.host, &io->src.auth, NULL, 0);
+					iolog(&io->rule, &state, OPERATION_ERROR, &rfromhost,
+					&io->dst.auth, &io->src.host, &io->src.auth, NULL, 0);
 				io->src.written += MAX(0, w);
+
+				if (s != io->src.s) /* socket temporarly created for redirect. */
+					close(s);
 			}
 			break;
 		}
@@ -1066,16 +1202,27 @@ doio(mother, io, rset, wset, flags)
 		}
 	}
 
-	/* don't care what direction/descriptors i/o was done over. */
-	time(&io->time);
+
+	if (bwused) {
+		io->time = timenow;
+
+		if (io->rule.bw != NULL) {
+			bwupdate(io->rule.bw, bwused, &io->time);
+		}
+	}
+
+	/* not saving rules used for udp, free after each usage. */
+	if (io->state.protocol == SOCKS_UDP)
+		bwfree(io->rule.bw);
 }
 
 static int
-io_rw(in, out, bad, buf, flag)
+io_rw(in, out, bad, buf, bufsize, flag)
 	struct sockd_io_direction_t *in;
 	struct sockd_io_direction_t *out;
 	int *bad;
-	char *buf;
+	void *buf;
+	size_t bufsize;
 	int flag;
 {
 	ssize_t r, w;
@@ -1086,7 +1233,7 @@ io_rw(in, out, bad, buf, flag)
 			flag &= ~MSG_OOB;
 
 	/* we receive oob inline. */
-	len = flag & MSG_OOB ? 1 : out->sndlowat;
+	len = MIN(bufsize, flag & MSG_OOB ? 1 : out->sndlowat);
 	if ((r = socks_recvfrom(in->s, buf, len, flag & ~MSG_OOB, NULL, NULL,
 	&in->auth)) <= 0) {
 		*bad = in->s;
@@ -1121,14 +1268,15 @@ io_getset(set)
 	fd_set *set;
 {
 	int i;
+	struct sockd_io_t *best, *evaluating;
 
-	for (i = 0; i < ioc; ++i)
+	for (i = 0, best = evaluating = NULL; i < ioc; ++i)
 		if (iov[i].allocated) {
 			if (FD_ISSET(iov[i].src.s, set))
-				return &iov[i];
+					evaluating = &iov[i];
 
 			if (FD_ISSET(iov[i].dst.s, set))
-				return &iov[i];
+				evaluating = &iov[i];
 
 			switch (iov[i].state.command) {
 				case SOCKS_BIND:
@@ -1139,15 +1287,19 @@ io_getset(set)
 
 				case SOCKS_UDPASSOCIATE:
 					if (FD_ISSET(iov[i].control.s, set))
-						return &iov[i];
+						evaluating = &iov[i];
 					break;
 
 				default:
 					break;
 			}
+
+			/* select the i/o that has least recently done i/o. */
+			if (best == NULL || timercmp(&evaluating->time, &best->time, <))
+				best = evaluating;
 		}
 
-	return NULL;
+	return best;
 }
 
 
@@ -1184,37 +1336,57 @@ io_finddescriptor(d)
 
 
 static int
-io_fillset(set, antiflags)
+io_fillset(set, antiflags, timenow)
 	fd_set *set;
 	int antiflags;
+	const struct timeval *timenow;
 {
+	const char *function = "io_fillset()";
 	int i, max;
 
 	FD_ZERO(set);
 
-	for (i = 0, max = -1; i < ioc; ++i)
-		if (iov[i].allocated) {
-			if (! (antiflags & iov[i].src.flags)) {
-				FD_SET(iov[i].src.s, set);
-				max = MAX(max, iov[i].src.s);
+	for (i = 0, max = -1; i < ioc; ++i) {
+		struct sockd_io_t *io = &iov[i];
+
+		if (io->allocated) {
+			if (io->rule.bw != NULL) {
+				struct timeval new_bwoverflow;
+
+				if (isbwoverflow(io->rule.bw, timenow, &new_bwoverflow) != NULL) {
+					if (!timerisset(&bwoverflow)
+					|| timercmp(&new_bwoverflow, &bwoverflow, <))
+						bwoverflow = new_bwoverflow;
+
+					/*
+					 * XXX this also means we won't catch errors on this
+					 * client for the duration.  Hopefully not a problem.
+					 */
+					continue;
+				}
 			}
 
-			if (! (antiflags & iov[i].dst.flags)) {
-				FD_SET(iov[i].dst.s, set);
-				max = MAX(max, iov[i].dst.s);
+			if (! (antiflags & io->src.flags)) {
+				FD_SET(io->src.s, set);
+				max = MAX(max, io->src.s);
 			}
 
-			switch (iov[i].state.command) {
+			if (! (antiflags & io->dst.flags)) {
+				FD_SET(io->dst.s, set);
+				max = MAX(max, io->dst.s);
+			}
+
+			switch (io->state.command) {
 				case SOCKS_BIND:
 				case SOCKS_BINDREPLY:
-					if (!iov[i].state.extension.bind)
+					if (!io->state.extension.bind)
 						break;
 					/* else: */ /* FALLTHROUGH */
 
 				case SOCKS_UDPASSOCIATE:
-					if (! (antiflags & iov[i].control.flags)) {
-						FD_SET(iov[i].control.s, set);
-						max = MAX(max, iov[i].control.s);
+					if (! (antiflags & io->control.flags)) {
+						FD_SET(io->control.s, set);
+						max = MAX(max, io->control.s);
 					}
 					break;
 
@@ -1222,49 +1394,82 @@ io_fillset(set, antiflags)
 					break;
 			}
 		}
+	}
 
 	return max;
 }
 
 static struct timeval *
-io_gettimeout(timeout)
+io_gettimeout(timeout, timenow)
 	struct timeval *timeout;
+	const struct timeval *timenow;
 {
-	time_t timenow;
+	const char *function = "io_gettimeout()";
 	int i;
 
-	if (allocated() == 0 || config.timeout.io == 0)
+	if (allocated() == 0)
 		return NULL;
 
-	timeout->tv_sec	= config.timeout.io;
+	if (socksconfig.timeout.io == 0 && !timerisset(&bwoverflow))
+		return NULL;
+
+	timeout->tv_sec	= socksconfig.timeout.io;
 	timeout->tv_usec	= 0;
 
-	time(&timenow);
-	for (i = 0; i < ioc; ++i)
-		if (!iov[i].allocated)
-			continue;
+	if (timerisset(timeout))
+		for (i = 0; i < ioc; ++i)
+			if (!iov[i].allocated)
+				continue;
+			else
+				timeout->tv_sec = MAX(0, MIN(timeout->tv_sec,
+				difftime(socksconfig.timeout.io,
+				(time_t)difftime((time_t)timenow->tv_sec,
+				(time_t)iov[i].time.tv_sec))));
+	else {
+		if (timerisset(&bwoverflow))
+			*timeout = bwoverflow;
 		else
-			timeout->tv_sec = MAX(0, MIN(timeout->tv_sec,
-			difftime(config.timeout.io, (time_t)difftime(timenow, iov[i].time))));
+			timeout = NULL;
+		return timeout;
+	}
+
+	if (timerisset(&bwoverflow)) {
+		struct timeval timetobw;
+		
+		if (timercmp(timenow, &bwoverflow, >))
+			/* CONSTCOND */ /* macro operation. */
+			timersub_hack(timenow, &bwoverflow, &timetobw);
+		else
+			timerclear(&timetobw);
+
+		if (timercmp(&timetobw, timeout, <)) {
+			*timeout = timetobw;
+		}
+	}
+
+#if 0
+slog(LOG_DEBUG, "%s: timeout = %d.%d",
+function, timeout->tv_sec, timeout->tv_usec);
+#endif
 
 	return timeout;
 }
 
 static struct sockd_io_t *
-io_gettimedout(void)
+io_gettimedout(timenow)
+	const struct timeval *timenow;
 {
 	int i;
-	time_t timenow;
 
-	if (config.timeout.io == 0)
+	if (socksconfig.timeout.io == 0)
 		return NULL;
 
-	time(&timenow);
 	for (i = 0; i < ioc; ++i)
 		if (!iov[i].allocated)
 			continue;
 		else
-			if (difftime(timenow, iov[i].time) >= config.timeout.io)
+			if (difftime((time_t)timenow->tv_sec, (time_t)iov[i].time.tv_sec)
+			>= socksconfig.timeout.io)
 				return &iov[i];
 
 	return NULL;
@@ -1309,7 +1514,7 @@ siginfo(sig)
 			slog(LOG_INFO, "%s <-> %s: idle %.0fs",
 			sockshost2string(&iov[i].src.host, srcstring, sizeof(srcstring)),
 			sockshost2string(&iov[i].dst.host, dststring, sizeof(dststring)),
-			difftime(timenow, iov[i].time));
+			difftime(timenow, (time_t)iov[i].time.tv_sec));
 		}
 
 }
