@@ -44,12 +44,12 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: sockd_socket.c,v 1.29 2001/02/06 15:59:13 michaels Exp $";
+"$Id: sockd_socket.c,v 1.32 2001/11/11 13:38:45 michaels Exp $";
 
 int
 sockd_bind(s, addr, retries)
 	int s;
-	const struct sockaddr *addr;
+	struct sockaddr *addr;
 	size_t retries;
 {
 /*	const char *function = "sockd_bind()"; */
@@ -62,7 +62,23 @@ sockd_bind(s, addr, retries)
 		if (tries++ > 0)
 			sleep(tries - 1);
 
-		if ((p = bind(s, addr, sizeof(*addr))) == 0)
+		/* LINTED pointer casts may be troublesome */
+		if (PORTISRESERVED(TOIN(addr)->sin_port) && socksconfig.compat.sameport) {
+			uid_t euid;
+
+			socks_seteuid(&euid, socksconfig.uid.privileged);
+			/* LINTED pointer casts may be troublesome */
+			p = bindresvport(s, TOIN(addr));
+			socks_reseteuid(socksconfig.uid.privileged, euid);
+		}
+		else if ((p = bind(s, addr, sizeof(*addr))) == 0) {
+			socklen_t addrlen;
+
+			addrlen = sizeof(*addr);
+			p = getsockname(s, addr, &addrlen);
+		}
+
+		if (p == 0)
 			break;
 		else {
 			/* non-fatal error? */
